﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOPC104Net;  // The namespace exposes the AIOPC104 Class interface (AIO167.dll wrapper)


namespace Sample0
{
    public partial class Form1 : Form
    {
        // Constants:
        public const Byte RLD = 0x80;
        public const Byte XRLD = 0x00;
        public const Byte YRLD = XRLD;
        public const Byte Rst_BP = 0x01;
        public const Byte Rst_CTR = 0x02;
        public const Byte Rst_FLAGS = 0x04;
        public const Byte Rst_E = 0x06;
        public const Byte Trf_PR_CTR = 0x08;
        public const Byte Trf_CTR_OL = 0x10;
        public const Byte Trf_PS0_PSC = 0x18;

        public const Byte CMR = 0xA0;
        public const Byte XCMR = 0x20;
        public const Byte YCMR = XCMR;
        public const Byte BINCnt = 0x00;
        public const Byte BCDCnt = 0x01;
        public const Byte NrmCnt = 0x00;
        public const Byte RngLmt = 0x02;
        public const Byte NRcyc = 0x04;
        public const Byte ModN = 0x06;
        public const Byte NQDX = 0x00;
        public const Byte QDX1 = 0x08;
        public const Byte QDX2 = 0x10;
        public const Byte QDX4 = 0x18;

        public const Byte IOR = 0xC0;
        public const Byte XIOR = 0x40;
        public const Byte YIOR = 0x00;
        public const Byte DisAB = 0x00;
        public const Byte EnAB = 0x01;
        public const Byte LCTR = 0x00;
        public const Byte LOL = 0x02;
        public const Byte RCTR = 0x00;
        public const Byte ABGate = 0x04;
        public const Byte CYBW = 0x00;
        public const Byte CPBW = 0x08;
        public const Byte CB_UPDN = 0x10;
        public const Byte IDX_ERR = 0x18;

        public const Byte IDR = 0xE0;
        public const Byte XIDR = 0x60;
        public const Byte YIDR = XIDR;
        public const Byte DisIDX = 0x00;
        public const Byte EnIDX = 0x01;
        public const Byte NIDX = 0x00;
        public const Byte PIDX = 0x02;
        public const Byte LIDX = 0x00;
        public const Byte RIDX = 0x04;

        public const double PI = 3.141593;

        public Int32[] previous_readings = new Int32[8]; // buffer data for display
        public Int32[] currentReadings = new Int32[8]; // buffer data for display
        public UInt32[] currentRes = new UInt32[8]; // buffer data for display

        // Base address of the Card:
        UInt32 baseAddr;  

        // Array of controls for GUI updates:
        public TextBox[] ChannelData = new TextBox[8]; // buffer data for display

        // Data used to discover cards in the system:
        UInt32 Status = 0;
        const UInt32 MAX_CARDS = 10;
        public Int32 NumCards = 0;

        // This is the basic data that we will be using to interface with the card:
        public Int32 CardNum = 0;   // The index of the one card we will be using 0
        public UInt32 Offset = 0;   // This the offset that will be used based on the base address of the card
            
        bool RunFlag;
        public bool TimerEnabled = false;
      
        public int i = 0;

        public struct TCardData 
	    {
		    public bool IsValid;
            public bool IsSelected;
            public UInt32 DeviceID;
            public UInt32 Base;     // This the result of findcards and the base address of the card
	    };
        public TCardData CardData; // only one struct
        

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();


        public Form1()
        {
            InitializeComponent();

            // Initialize default values:

            // Arrays of the controls:
            ChannelData[0] = textBoxVolts0;
            ChannelData[1] = textBoxVolts1;
            ChannelData[2] = textBoxVolts2;
            ChannelData[3] = textBoxVolts3;
            ChannelData[4] = textBoxVolts4;
            ChannelData[5] = textBoxVolts5;
            ChannelData[6] = textBoxVolts6;
            ChannelData[7] = textBoxVolts7;
                  
            // Check device info:
            UInt32 iNumCards = AIOPC104.GetNumCards();

            textBoxStatus.Text = "This sample demonstrates basic operation of the board.";
            
            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // Set the timer interval in miliseconds and start 55ms min resolution:
            myTimer.Interval = 100;
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {

        }
       
        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:
            myTimer.Stop();

            UpdateGUIState();

            myTimer.Enabled = true;
        }

        private void UpdateGUIState()
        {
            // Get data and display:
	        String readString;

	        for (int count = 0; count < 8; count++)
	        {
		        previous_readings[count] = currentReadings[count];
		        currentReadings[count] = Read_7266_OL( (uint)(baseAddr + (count * 2)) );
		        if (currentReadings[count] != previous_readings[count])
		        {
			        //change = true;
                    readString = currentReadings[count].ToString();
                    ChannelData[count].Text = readString;
		        }
	        }
        }
      
        private void btnPerformIO_Click(object sender, EventArgs e)
        {
            String displayString; 
            
            if (TimerEnabled)
	        {
		        // Stop timer:
                TimerEnabled = false;
                myTimer.Stop();
                btnPerformIO.Text = "Perform I/O";
	        }
	        else
	        {
                // Get data:
                for (int count = 0; count < 8; count++)
                {
                    previous_readings[count] = Read_7266_OL((uint)(baseAddr + (count * 2)));
                    currentReadings[count] = previous_readings[count];
                    displayString = currentReadings[count].ToString();
                    ChannelData[count].Text = displayString;
                }
                
                // Start timer:
                TimerEnabled = true;
                myTimer.Enabled = true;
                btnPerformIO.Text = "Stop I/O";
	        }

        }
        
        private void btnExit_Click(object sender, EventArgs e)
        {
            // Right now this is only called on button click not window close ALtF4 etc:

            // Kill the timer:
            myTimer.Stop();
            myTimer.Enabled = false;
            myTimer.Dispose();

            // Close the form and Application:
            this.Close();
        }

        private void btnInit_Click(object sender, EventArgs e)
        {
            String address;

	        address = textBoxAddress.Text;
            baseAddr = uint.Parse(address, System.Globalization.NumberStyles.HexNumber);

	        Init_7266(baseAddr);
            AIOPC104.OutPortB(baseAddr + 0x14, 1);

            btnInit.Enabled = false;
            btnPerformIO.Enabled = true;
        }

        private void resetCount(uint channel)
        {
            String address;
            address = textBoxAddress.Text;
            uint baseAddr = Convert.ToUInt32(address); 
            
            AIOPC104.OutPortB(baseAddr + 1 + (2 * channel), Rst_CTR);
        }

        private void Init_7266(UInt32 Addr)
        {           
            UInt32 XCt;
            UInt32 YCt;

            for (int count = 0; count < 4; count++)
            {
	            XCt = 0xFFFFFF;
                YCt = 0xFFFFFF;

	            //Setup IOR reg
	            AIOPC104.OutPortB(Addr + 1, IOR + DisAB + LOL + ABGate + CYBW);

	            //Setup RLD reg
                AIOPC104.OutPortB(Addr + 1, RLD + Rst_BP + Rst_FLAGS);
                AIOPC104.OutPortB(Addr + 0, 0x0);
                AIOPC104.OutPortB(Addr + 2, 0x0);
                AIOPC104.OutPortB(Addr + 1, RLD + Rst_E + Trf_PS0_PSC);
                AIOPC104.OutPortB(Addr + 1, RLD + Rst_BP + Rst_CTR);

	            //Setup IDR reg
                AIOPC104.OutPortB(Addr + 1, IDR + EnIDX + NIDX + LIDX);

	            //Setup CMR reg
                AIOPC104.OutPortB(Addr + 1, CMR + BINCnt + ModN + QDX1);

	            //Setup PR reg for modulo N counter to XCt
                AIOPC104.OutPortB(Addr + 0, (byte) XCt);
                XCt >>= 8;
                AIOPC104.OutPortB(Addr + 0, (byte) XCt);
                XCt >>= 8;
                AIOPC104.OutPortB(Addr + 0, (byte) XCt);

	            //Setup PR reg for modulo N counter to YCt
                AIOPC104.OutPortB(Addr + 2, (byte) YCt);
                YCt >>= 8;
                AIOPC104.OutPortB(Addr + 2, (byte) YCt);
                YCt >>= 8;
                AIOPC104.OutPortB(Addr + 2, (byte) YCt);

	            //Enable counters
                AIOPC104.OutPortB(Addr + 1, IOR + EnAB);

                Addr += 4;
            }          
        }

        private int Read_7266_OL( UInt32 Addr)
        {           
            // Read 4 bytes into one 32 bit int:
            int result = 0;

            //UInt16[] reading = new UInt16[4];
            byte[] reading = new byte[3];
  	        
            AIOPC104.OutPortB(Addr + 1, 0x10);	// reset address pointer
	        AIOPC104.OutPortB(Addr + 1, 0x01);	// command to latch counter
            AIOPC104.OutPortB(Addr + 1, RLD + Rst_BP + Trf_CTR_OL);
            
            reading[0] = (byte)AIOPC104.InPortB(Addr);
            reading[1] = (byte)AIOPC104.InPortB(Addr);
            reading[2] = (byte)AIOPC104.InPortB(Addr);

            if ((reading[2] & 0x80) != 0)
            {
                reading[2] = (byte)~reading[2];
                reading[1] = (byte)~reading[1];
                reading[0] = (byte)~reading[0];
                
                result = (reading[2] << 16) | (reading[1] << 8) | reading[0];
                result = (result * -1) - 1;
            }
            else
            {
                result = (reading[2] << 16) | (reading[1] << 8) | reading[0];
            }

            return (result);
        }

        private UInt16 Get_7266_Flags(UInt32 Addr)
        {
            return(AIOPC104.InPortB(Addr+1));
        }

    }
}
